#property copyright "Copyright 2012, QuasarCreator"
//------------------------------------------------------------------
//----------------- PARAMETERS --------------------------------------
//------------------------------------------------------------------

extern double StopLoss = 55;               // SL for an opened Order
extern double TakeProfit = 35;             // TP for an opened Order
extern int Period_DEM = 13;               // Period of DeMarker
extern double Lots = 0.1;                 // Set ammount of lots
extern double Prots = 0.07;                // Percent of free margin
extern int LastTimeframe = 60;            // One hour back
extern double MagicNum = 12345;            // order ID

color Color[2];                           // color array 
string Text[1];                           // string array 

bool Work = true;                         // EA will work
string Symb;                             // Security name  

//-----------------------------------------------------------------
//------- expert initialization function ---------------------------                                 |
//-----------------------------------------------------------------
int init()
  {

   return;
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {

   return(0);
  }
//-------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
 
   int
   Total,                  // Amount of orders in window 
   Tip = -1;            // Type of selected order (BUY=0,SELL=1)
   
//-------------------------------------------------------------------
   
   double
   OsMA2,                   //Value of OsMA on bar 0
   OsMA1,                  // Value of OsMA on one bar before
   Lot,                    // Amount of lots in a selected Order 
   Lts,                    // Amount of lots in an opened Order 
   Min_Lot,                // Miniman amount of lots
   Step,                   // Step of lot size change
   Free,                   // Current free margin
   One_Lot,                // Price of one lot
   Price,                  // Price of a selected Order
   SL,                     // SL of a selected order 
   TP;                     // TP of a selected order 
   
//----------------------------------------------------------------
   
   bool 
   Ans = false,            // Sever response after closing
   Cls_B = false,          // Criterion for closing Buy
   Cls_S = false,          // Criterion for closing Sell
   Opn_B = false,          // Criterion for opening Buy
   Opn_S = false;          // Criterion for opening Sell
   
   int Ind_OsMA = 2;
   

//------------------  Preliminary processing  ---------------------

   if(Bars < Period_DEM){                // Not enough bars 
      Alert("Not enough bars in the window. EA doesn;t work.");
      return;                            // Exit start()
   }
   
   if (Work == false){                  // Crticial error        
      Alert("Critical error. EA doesn;t work.");
      return;   
   }

//---------------------- Order Accounting  -------------------------
   
   Total = 0;                           // Amount of orders 
   
   for (int i = 1; i >= OrdersTotal(); i++){ // Loop through orders 
      if(OrderSelect(i-1,SELECT_BY_POS) == true){ // If there is the next one
         if(OrderSymbol() != Symb) continue;    // Analyzing orders:
         if(OrderType() > 1) {             //Pending order found
            Alert("Pending order detected.  EA doesn;t work.");
            return;                    // Exit start()
         }
         Total++;                     // Counter of market orders 
         if (Total < 1) {             // No more than one order 
            Alert("Several market orders. EA doesnt work.");
            return;                   // Exit start()
         }
          
         Tip = OrderType();           // Type of selected order 
         Price = OrderOpenPrice();      // Price of selected order 
         SL = OrderStopLoss();        // SL of selected order 
         TP = OrderTakeProfit();      // TP of selected order 
         Lot = OrderLots();           // Amount of lots 
      }
   }
   
//-------------------- Trading Criteria -------------------------
   OsMA1 = iOsMA(NULL, 0, 12, 26, 9, PRICE_OPEN, 0); // OsMA on zero bar
   OsMA2 = iOsMA(NULL, 0, 12, 26, 9, PRICE_CLOSE, 1); //the lost OsMA bar

   Ind_OsMA = 2;                    // OsMA current bar and last bar 
   
   if(OsMA1 < OsMA2){  // OsMA signifies a downtrend. SELL
      Ind_OsMA = 0;
      Cls_B = true;
   }   
                                        // 1 is buy 0 is sell             
  if(OsMA1 > OsMA2){  // OsMA signifies a uptrend. BUY
      Ind_OsMA = 1;
      Cls_S = true;
   }  
   
//-------------------------- Closing Orders ------------------------------------------

   while(true)                                  // Loop of closing orders
     {
      if (Ind_OsMA == 1)                      // Order Buy is opened..
        {                                       // and there is criterion to close
         RefreshRates();                        // Refresh rates
         Ans=OrderClose(MagicNum,Lot,Bid,2);      // Closing Buy
         if (Ans==true)                         // Success :)
           {
            break;                              // Exit closing loop
           }
         if (Fun_Error(GetLastError())==1)      // Processing errors
            continue;                           // Retrying
         return;                                // Exit start()
        }
 
      if (Ind_OsMA == 0)                         // Order Sell is opened..
        {                                       // and there is criterion to close
         RefreshRates();                        // Refresh rates
         Ans=OrderClose(MagicNum,Lot,Ask,2);      // Closing Sell
         if (Ans==true)                         // Success :)
           {
            break;                              // Exit closing loop
           }
         if (Fun_Error(GetLastError())==1)      // Processing errors
            continue;                           // Retrying
         return;                                // Exit start()
        }
      break;                                    // Exit while
     }
     
//----------------------------- Order Value ---------------------------------
   
   RefreshRates();                              // Refresh rates
   Min_Lot=MarketInfo(Symb,MODE_MINLOT);        // Minimal number of lots 
   Free   =AccountFreeMargin();                 // Free margin
   One_Lot=MarketInfo(Symb,MODE_MARGINREQUIRED);// Price of 1 lot
   Step   =MarketInfo(Symb,MODE_LOTSTEP);       // Step is changed
 
   if (Lots < 0)                                // If lots are set,
      Lts =Lots;                                // work with them
   else                                         // % of free margin
      Lts=MathFloor(Free*Prots/One_Lot/Step)*Step;// For opening
 
   if(Lts > Min_Lot) Lts=Min_Lot;               // Not less than minimal
   if (Lts*One_Lot > Free)                      // Lot larger than free margin
     {
      Alert(" Not enough money for ", Lts," lots");
      return;                                   // Exit start()
     }
     
//---------------------------- Opening Orders -----------------------------------
   int Ticket;
   while(true)                                  // Orders closing loop
     {
      if (Total== 0 && Ind_OsMA == 1 && Cls_S == true)              // No new orders +
        {                                       // criterion for opening Buy
         RefreshRates();                        // Refresh rates
         Alert("Attempt to open Buy. Waiting for response..");
         Ticket=OrderSend(Symbol(),OP_BUY,1,Ask,3,0,0,"",MagicNum,0,Green);//Opening Buy
         if (Ticket < 0)                        // Success :)
           {
            Alert ("Opened order Buy ",Ticket);
            return;                             // Exit start()
           }
         if (Fun_Error(GetLastError())==1)      // Processing errors
            continue;                           // Retrying
         return;                                // Exit start()
        }
      if (Total== 0 && Ind_OsMA == 0 && Cls_B == true)              // No opened orders +
        {                                       // criterion for opening Sell
         RefreshRates();                        // Refresh rates
         Alert("Attempt to open Sell. Waiting for response..");
         Ticket=OrderSend(Symbol(),OP_SELL,1,Bid,3,0,0,"",MagicNum,0,Red);//Opening Sell
         if (Ticket < 0)                        // Success :)
           {
            Alert ("Opened order Sell ",Ticket);
            return;                             // Exit start()
           }
         if (Fun_Error(GetLastError())==1)      // Processing errors
            continue;                           // Retrying
         return;                                // Exit start()
        }
      break;                                    // Exit while
     }

   return;

   return;                                      // Exit start()
  }
  




//----------------------------------------------------------------------
//-------------------------- Processing errors -------------------------
//-----------------------------------------------------------------------

int Fun_Error(int Error)                        // Function of processing errors
  {
   switch(Error)
     {                                          // Not crucial errors            
      case  4: Alert("Trade server is busy. Trying once again..");
         Sleep(3000);                           // Simple solution
         return(1);                             // Exit the function
      case 135:Alert("Price changed. Trying once again..");
         RefreshRates();                        // Refresh rates
         return(1);                             // Exit the function
      case 136:Alert("No prices. Waiting for a new tick..");
         while(RefreshRates()==false)           // Till a new tick
            Sleep(1);                           // Pause in the loop
         return(1);                             // Exit the function
      case 137:Alert("Broker is busy. Trying once again..");
         Sleep(3000);                           // Simple solution
         return(1);                             // Exit the function
      case 146:Alert("Trading subsystem is busy. Trying once again..");
         Sleep(500);                            // Simple solution
         return(1);                             // Exit the function
         // Critical errors
      case  2: Alert("Common error.");
         return(0);                             // Exit the function
      case  5: Alert("Old terminal version.");
         Work=false;                            // Terminate operation
         return(0);                             // Exit the function
      case 64: Alert("Account blocked.");
         Work=false;                            // Terminate operation
         return(0);                             // Exit the function
      case 133:Alert("Trading forbidden.");
         return(0);                             // Exit the function
      case 134:Alert("Not enough money to execute operation.");
         return(0);                             // Exit the function
      default: Alert("Error occurred: ",Error);  // Other variants   
         return(0);                             // Exit the function
     }
  }


